//Simple Program to demonstate apparent bug regarding SDL_VIDEORESIZE for window restore operation
//Operating System - Windows Vista Home Premium, Service Pack 1
//32 bit Operating System
//PC: Dell Studio Hybrid 140G
//Compiler - Microsoft Visual C++ 2008 Express Edition
//Date - December 14, 2009
//
//Submitted by Chris Jacobsen (datahead8888)
//Email address (eliminate the spaces)::::
//cjj _ 009 @ yahoo.com
//
//----------------------------------------------------------------------
//Forum post made at http://forums.libsdl.org/viewtopic.php?t=5291
//
//I've been working on an SDL/OpenGL program, that among other things, must deal with resizing events in order to adjust the aspect ratio.
//It doesn't always seem to get the SDL_VIDEORESIZE event when it should, causing the aspect ratio to not be adjusted as needed.
//I've run it in debug mode and made these observations:
//*When it initially starts up, if I maximize the window, it receives the SDL_VIDEORESIZE event as needed.
//*If, after starting up the the application and maximizing the window, I then restore the window by double clicking the title bar, it does NOT receive the SDL_VIDEORESIZE event.
//*I can repeat the last two steps, and it will get continue to get the SDL_VIDEORESIZE on the maximize but not get one on the restore.
//*If I then do a slight adjustment to the width or height of the window, it will get the SDL_VIDEORESIZE event.
//*From then on, if I do restore operations to the window, the SDL_VIDEORESIZE event will be caught properly.
//
//I then did some research, and found this documentation:
//http://www.libsdl.org/release/changes-1.2.html
//Under SDL 1.2.10 Release Notes, section "Windows Notes", it reads:
//*An SDL_VIDEORESIZE event is properly sent when the SDL window is maximized and restored.
//This tells me a bug fix was put in for this issue as of SDL Version 1.2.10
//I was using SDL Version SDL-1.2.13, on Windows Vista.
//Thinking that the version number was my problem, I've now changed it to version SDL-1.2.14.
//The problem persists. Since both versions I tried were past version 10, I wouldn't think this issue should have ever happened.
//
//-----------------------------------------------------------------------------------------
//This is a simple, quick program whose sole purpose is to demonstrate the apparent issue
//It implements a rotating pyramid.  If you resize, it will adjust the aspect ratio.
//If you press escape, it will quit.
//
//The 5 bulleted steps above for producing the problem apply to this program
//It's generally pretty obvious if you have the problem, because the aspect ratio will be incorrect.
//I've included a Win32 alert function that is currently commented out if you need a better utility for seeing
//when the SDL_RESIZE_EVENT is called, though you may just want to use the debugger.
//If you choose to use the message box, note that you must click it to resume the program.
//
//Let me know via the forum post if you need additional information.

#include "SDL.h"  
#include "SDL_opengl.h"  
//#include "Windows.h"  //Win32 only.  Used to implement message box in alert funciton at bottom of code

#include <string>

//void alert(std::string text);
void setupPerspective(int xSize, int ySize);
void renderPyramid();  


int main(int argc, char **argv)
{
	//Hard coded X and Y dimensions of screen.
	const int screenXSize = 600;
	const int screenYSize = 450;
	
	//SDL Open GL Intaliazation
	SDL_Init(SDL_INIT_EVERYTHING);  
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);

	SDL_Surface * cont = SDL_SetVideoMode(screenXSize, screenYSize, 0, SDL_OPENGL | SDL_RESIZABLE);  //Full screen!
	
	//Open GL Setup
	glEnable(GL_DEPTH_TEST);  
	glShadeModel(GL_SMOOTH);  

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	//1st time aspect ratio
	setupPerspective(screenXSize, screenYSize);
	
	bool running = true;  
	SDL_Event theEvent;   

	//Simple message looop
	while (running)
	{
		while (SDL_PollEvent(&theEvent))
		{

			switch (theEvent.type)    
			{
				case SDL_KEYDOWN:
					if (theEvent.key.keysym.sym == SDLK_ESCAPE)  
						running = false;
					break;		          
				case SDL_QUIT:
					running = false;
					break;
				case SDL_VIDEORESIZE:
					setupPerspective(theEvent.resize.w, theEvent.resize.h);
					//alert("SDL_VIDEORESIZE event processed");
					break;
			} //switch (theEvent.type)
			
			
			
		} //while SDL_PollEvent()

		renderPyramid();
		SDL_GL_SwapBuffers();
	}

	//Application cleanup and termination
	SDL_Quit();
	return 0;
}

//This function takes x and y dimensions for the screen size and sets up the perspective for the scene
void setupPerspective(int xSize, int ySize)
{
	//Calculate the aspect ratio
	float ar = (float) xSize / ySize;

	//Projection Matrix
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();

	//Viewport and perspective
	glViewport(0, 0, xSize, ySize);
	gluPerspective(40.0, ar, 0.1, 50.0);

	//Model View matrix
	glMatrixMode(GL_MODELVIEW);
}

//Render the parymid on screen
void renderPyramid()
{
	//Quick and Dirty pyramid rotation
	static int i = -1;   
	i = (i + 1) % 360;
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	
	//Positioning and orientation of pyramid
	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -7.0f);
	glRotatef(i, 0.0f, 1.0f, 0.0f);
	
	//Render pyramid
	glBegin(GL_TRIANGLES);
		//Triangle 1
		glColor3f(1.0f, 0.0f, 0.0f);     
		glVertex3f(-1.0f, 0.0f, 0.0f);  
		glColor3f(0.0f, 1.0f, 0.0f);
		glVertex3f(0.0f, 1.0f, 0.0f);
		glColor3f(0.0f, 0.0f, 1.0f);
		glVertex3f(1.0f, 0.0f, 0.0f);
		//Triangle 2
		glColor3f(1.0f, 0.5f, 0.0f);     
		glVertex3f(1.0f, 0.0f, 0.0f);  
		glColor3f(0.5f, 1.0f, 0.0f);
		glVertex3f(0.0f, 1.0f, 0.0f);
		glColor3f(0.5f, 0.5f, 1.0f);
		glVertex3f(0.0f, 0.0f, -1.0f);
		//Triangle 3
		glColor3f(1.0f, 0.5f, 0.0f);     
		glVertex3f(0.0f, 0.0f, -1.0f);  
		glColor3f(0.5f, 1.0f, 0.0f);
		glVertex3f(-1.0f, 0.0f, 0.0f);
		glColor3f(0.5f, 0.5f, 1.0f);
		glVertex3f(0.0f, 1.0f, 0.0f);

	glEnd();

	glFlush();
	
	
}

//Win32 utlity method to allow message boxes
//Note that if you use this, you must close the message box to resume the program
//void alert(std::string text)
//{
//	MessageBoxA(NULL, text.c_str(), "Info", MB_OK);
//}